# Kaitai file for AMICALSAT satellite
#
# based on  AmicalSat downlinks technicals informations document
#
# Author  : C. Mercier for AMSAT-Francophone
#
# Telemetry based on APRS  
#
# Version : 0.1 
#
#



meta:
  id: frame_tlm_amicalsat
  title: AMICALSAT TLM Frame
  file-extension: .data
  license:  LGPL V2
  ks-version: 0.8
  endian: be

 
  

seq:
  - id: frame_tlm_entry_sat
    type: ax25_frame
    doc-ref: 'https://www.tapr.org/pub_ax25.html'

types:
  ax25_frame:
    seq:
    - id: ax25_header
      type: ax25_header
    - id: payload
      type: i_frame

  ax25_header:
    seq:
      - id: dest_callsign_raw
        type: dest_callsign_raw
      - id: dest_ssid_raw
        type: u1
      - id: src_callsign_raw
        type: src_callsign_raw
      - id: src_ssid_raw
        type: u1
      - id: ctl
        type: u1
    instances:
      src_ssid:
        value: (src_ssid_raw & 0x0f) >> 1
      dest_ssid:
        value: (dest_ssid_raw & 0x0f) >> 1
  dest_callsign_raw:
    seq:
      - id: dest_callsign_ror
        process: ror(1)
        size: 6
        type: dest_callsign
  src_callsign_raw:
    seq:
      - id: src_callsign_ror
        process: ror(1)
        type: src_callsign
        size: 6
  dest_callsign:
    seq:
      - id: dest_callsign
        type: str
        encoding: ASCII
        size: 6
  src_callsign:
    seq:
      - id: src_callsign
        type: str
        encoding: ASCII
        size: 6

  i_frame:
    seq:
      - id: pid
        type: u1
      - id: field
        type: str
        encoding: ASCII
        terminator: 0x3D
      - id: tlm_area
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: tlm_type
        type: str
        encoding: ASCII
        terminator: 0x3B
 
      - id: tlm_area_switch
        type:
          switch-on: tlm_area
          cases: 
            '"M1"': m1_type
            '"A1"': a1_type               
            '"EM"': emer_type
            '"ER"': emer_type  
            '"V1"': v1_u2_type
            '"U2"': v1_u2_type
            '"CU_R"' : cu_type                            
            '"CU_L"' : cu_type
      
        
        
  m1_type:
    seq:       
      - id: time_stamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: tlm_sw
        type:
          switch-on:  _parent.tlm_type
          cases: 
            '"LOG"': log_type
            '"FLAGS"': flag_type
                        
      - id: ax25_info
        size-eos: true


  a1_type:
    seq:       
      - id: tlm_sw
        type:
          switch-on:  _parent.tlm_type
          cases: 
            '"FLAGS"': a1_flag_type
            '"MAG"': a1_mag_type
            '"GYRO"' : a1_gyro_type
            '"POSITION"' : a1_position_type

  emer_type:
    seq:       
      - id: times_tamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: tlmsw
        type:
          switch-on:  _parent.tlm_type
          cases: 
            '"LOG"': em_log_type
            '"MN"': emmn_type

  v1_u2_type:
    seq:       
      - id: times_tamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: tlmsw
        type:
          switch-on:  _parent.tlm_type
          cases: 
            '"RL"': rl_type
            '"MS"': ms_type 

  cu_type:
     seq:       
      - id: times_tamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: tlmsw
        type:
          switch-on:  _parent.tlm_type
          cases: 
            '"ONYX SENSOR T"': onyx_type
            '"Log"': cu_log__type  

#M1;LOG;[Timestamp];[Boot number];[Up time];[CPU voltage];[CPU temperature]     
  log_type:
    seq:
      - id: boot_number
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: up_time
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: cpu_voltage
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: cpu_temperature
        type: str
        encoding: ASCII
        size-eos: true
        
#M1;FLAGS;[Timestamp];[Hex flags]
# to do         
  flag_type:
    seq:
      - id: hex_flags
        type: str
        encoding: ASCII
        size-eos: true
        
        
#A1;FLAGS;[mode];[flags];[faults]      

  a1_flag_type:
    seq:
      - id: mode
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: a1_flags
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: faults
        type: str
        encoding: ASCII
        size-eos: true
        
#A1;MAG;[Current timestamp];[MagX];[MagY];[MagZ]
  a1_mag_type:
    seq:
      - id: time_stamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: mag_x
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: mag_y
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: mag_z
        type: str
        encoding: ASCII
        size-eos: true

#A1;GYRO;[Current timestamp];[GyroX];[GyroY];[GyroZ]
  a1_gyro_type:
    seq:
      - id: time_stamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: giro_x
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: giro_y
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: giro_z
        type: str
        encoding: ASCII
        size-eos: true

#A1;POSITION;[Current timestamp];[Latitude];[Longitude]
  a1_position_type:
    seq:
      - id: time_stamps
        type: str
        encoding: ASCII
        terminator: 0x3B     
      - id: latitude
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: longitude
        type: str
        encoding: ASCII
        size-eos: true

#[EM/ER];MN;[Timestamp];[V in];[V solar];[I in];[P in];[P peak];[T cpu];[V cpu]

  emmn_type:
    seq:
      - id: v_in
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: v_solar
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: i_in
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: p_in
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: p_peak
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: t_cpu
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: v_cpu
        type: str
        encoding: ASCII
        size-eos: true
        
#EM;LOG;[Timestamp];[Boot number];[Input voltage];[Input current];[Input power];[Peak Power];[Solar panel voltage]
  em_log_type:
    seq:
      - id: boot_number
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: input_voltage
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: input_current
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: input_power
        type: str
        encoding: ASCII
        terminator: 0x3B  
      - id: peak_power
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: solar_panel_voltage
        type: str
        encoding: ASCII
        size-eos: true          

#[ V1/U2];RL;[Timestamp];[CPU voltage];[Battery voltage];[CPU temperature];[Amplifier temperature];[Flags]

  rl_type:
     seq:
      - id: cpu_voltage
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: battery_voltage
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: cpu_temperature
        type: str
        encoding: ASCII
        terminator: 0x3B 
      - id: amplifier_temperature
        type: str
        encoding: ASCII
        terminator: 0x3B	
# a revoir - passer en hexa pour le flag
      - id: flags
        type: str
        encoding: ASCII
        size-eos: true
#[V1/U2];MS;[Timestamp];[Current rssi];[Latch rssi];[AFC offset ]

  ms_type:
     seq:
      - id: current_rssi
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: latch_rssi
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: a_f_c_offset 
        type: str
        encoding: ASCII
        size-eos: true

#[CU_R/CU_L];LOG;[Timestamp];[CPU voltage];[CPU temperature];[flags ]

  cu_log__type:
    seq:   

      - id: cpu_voltage
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: cpu_temperature
        type: str
        encoding: ASCII
        terminator: 0x3B 
      - id: cu_flags 
        type: str
        encoding: ASCII
        size-eos: true

#[CU_R/CU_L];ONYX SENSOR T;[Timestamp];[Return value]

  onyx_type:
      seq:
      - id: time_stamp
        type: str
        encoding: ASCII
        terminator: 0x3B
      - id: return_value
        type: str
        encoding: ASCII
        size-eos: true     